#!/usr/bin/python3

import configparser
import pathlib
import struct

import msgspec

from .validate import (
    ByteSignature,
    Code,
    GameConfDict,
    IntLiteral,
    LinuxBinary,
    PlatformBinary,
    WindowsBinary,
    convert_types,
)


def main() -> None:
    # TODO implement
    p = pathlib.Path("tf2.gamedata.ini")

    config = configparser.ConfigParser()
    config.read(p)

    entries = msgspec.convert(
        {s: config[s] for s in config.sections()},
        type=GameConfDict,
        dec_hook=convert_types(ByteSignature, Code, IntLiteral, struct.Struct, pathlib.Path),
    )

    linux_target = LinuxBinary(pathlib.Path("server_srv.so"))
    windows_target = WindowsBinary(pathlib.Path("server.dll"))

    linux_target.path = pathlib.Path("232250/tf/bin/server_srv.so")
    windows_target.path = pathlib.Path("232250/tf/bin/server.dll")

    candidate_targets: list[PlatformBinary] = [linux_target, windows_target]

    for name, entry in entries.items():
        target = entry.get_target_match(candidate_targets)
        assert target, f"No binary match for {entry.target}"
        try:
            for key, result in entry.process(target).items():
                print("- [OK]", key.substitute(name=name), "=", result)
        except Exception as e:
            print("- [FAIL]", name, f"({type(e).__name__}):", e)


if __name__ == "__main__":
    main()
